@@ -25,7 +25,8 @@ import functools
 from botocore.exceptions import NoCredentialsError
 from botocore.utils import normalize_url_path
 from botocore.compat import HTTPHeaders
-from botocore.compat import quote, unquote, urlsplit
+from botocore.compat import quote, unquote, urlsplit, parse_qs, urlencode
+from botocore.compat import urlunsplit
 
 logger = logging.getLogger(__name__)
 
@@ -195,7 +196,7 @@ class SigV4Auth(BaseSigner):
                 elif len(q) == 1:
                     quoted_qsa.append('%s=' % quote(q[0], safe='-_.~'))
             if len(quoted_qsa) > 0:
-                quoted_qsa.sort(key=itemgetter(0))
+                quoted_qsa.sort()
                 buf += '&'.join(quoted_qsa)
         return buf
 
@@ -298,7 +299,7 @@ class SigV4Auth(BaseSigner):
         self.timestamp = now.strftime('%Y%m%dT%H%M%SZ')
         # This could be a retry.  Make sure the previous
         # authorization header is removed first.
-        self._add_headers_before_signing(request)
+        self._modify_request_before_signing(request)
         canonical_request = self.canonical_request(request)
         logger.debug("Calculating signature using v4 auth.")
         logger.debug('CanonicalRequest:\n%s', canonical_request)
@@ -307,6 +308,9 @@ class SigV4Auth(BaseSigner):
         signature = self.signature(string_to_sign)
         logger.debug('Signature:\n%s', signature)
 
+        self._inject_signature_to_request(request, signature)
+
+    def _inject_signature_to_request(self, request, signature):
         l = ['AWS4-HMAC-SHA256 Credential=%s' % self.scope(request)]
         headers_to_sign = self.headers_to_sign(request)
         l.append('SignedHeaders=%s' % self.signed_headers(headers_to_sign))
@@ -314,7 +318,7 @@ class SigV4Auth(BaseSigner):
         request.headers['Authorization'] = ', '.join(l)
         return request
 
-    def _add_headers_before_signing(self, request):
+    def _modify_request_before_signing(self, request):
         if 'Authorization' in request.headers:
             del request.headers['Authorization']
         if 'Date' not in request.headers:
@@ -325,7 +329,7 @@ class SigV4Auth(BaseSigner):
 
 class S3SigV4Auth(SigV4Auth):
 
-    def _add_headers_before_signing(self, request):
+    def _modify_request_before_signing(self, request):
         super(S3SigV4Auth, self)._add_headers_before_signing(request)
         request.headers['X-Amz-Content-SHA256'] = self.payload(request)
 
@@ -334,6 +338,98 @@ class S3SigV4Auth(SigV4Auth):
         return path
 
 
+class SigV4QueryAuth(SigV4Auth):
+    DEFAULT_EXPIRES = 3600
+
+    def __init__(self, credentials, service_name, region_name,
+                 expires=DEFAULT_EXPIRES):
+        super(SigV4QueryAuth, self).__init__(credentials, service_name,
+                                               region_name)
+        self._expires = expires
+
+    def _modify_request_before_signing(self, request):
+        # This is our chance to add additional query params we need
+        # before we go about calculating the signature.
+        request.headers = {}
+        request.method = 'GET'
+        # Note that we're not including X-Amz-Signature.
+        # From the docs: "The Canonical Query String must include all the query
+        # parameters from the preceding table except for X-Amz-Signature.
+        auth_params = {
+            'X-Amz-Algorithm': 'AWS4-HMAC-SHA256',
+            'X-Amz-Credential': self.scope(request),
+            'X-Amz-Date': self.timestamp,
+            'X-Amz-Expires': self._expires,
+            'X-Amz-SignedHeaders': 'host',
+        }
+        if self.credentials.token is not None:
+            auth_params['X-Amz-Security-Token'] = self.credentials.token
+        # Now parse the original query string to a dict, inject our new query
+        # params, and serialize back to a query string.
+        url_parts = urlsplit(request.url)
+        # parse_qs makes each value a list, but in our case we know we won't
+        # have repeated keys so we know we have single element lists which we
+        # can convert back to scalar values.
+        query_dict = dict([(k, v[0]) for k, v in parse_qs(url_parts.query).items()])
+        # The spec is particular about this.  It *has* to be:
+        # https://<endpoint>?<operation params>&<auth params>
+        # You can't mix the two types of params together, i.e just keep doing
+        # new_query_params.update(op_params)
+        # new_query_params.update(auth_params)
+        # urlencode(new_query_params)
+        operation_params = ''
+        if request.data:
+            # We also need to move the body params into the query string.
+            # request.data will be populated, for example, with query services
+            # which normally form encode the params into the body.
+            # This means that request.data is a dict() of the operation params.
+            query_dict.update(request.data)
+            request.data = ''
+        if query_dict:
+            operation_params = urlencode(query_dict) + '&'
+        new_query_string = operation_params + urlencode(auth_params)
+        # url_parts is a tuple (and therefore immutable) so we need to create
+        # a new url_parts with the new query string.
+        # <part>   - <index>
+        # scheme   - 0
+        # netloc   - 1
+        # path     - 2
+        # query    - 3  <-- we're replacing this.
+        # fragment - 4
+        p = url_parts
+        new_url_parts = (p[0], p[1], p[2], new_query_string, p[4])
+        request.url = urlunsplit(new_url_parts)
+
+    def _inject_signature_to_request(self, request, signature):
+        # Rather than calculating an "Authorization" header, for the query
+        # param quth, we just append an 'X-Amz-Signature' param to the end
+        # of the query string.
+        signed_headers = self.signed_headers(self.headers_to_sign(request))
+        request.url += (
+            '&X-Amz-Signature=%s' % (signature,))
+
+
+class S3SigV4QueryAuth(SigV4QueryAuth):
+    """S3 SigV4 auth using query parameters.
+
+    This signer will sign a request using query parameters and signature
+    version 4, i.e a "presigned url" signer.
+
+    Based off of: http://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html
+
+    """
+    def _normalize_url_path(self, path):
+        # For S3, we do not normalize the path.
+        return path
+
+    def payload(self, request):
+        # From the doc link above:
+        # "You don't include a payload hash in the Canonical Request, because
+        # when you create a presigned URL, you don't know anything about the
+        # payload. Instead, you use a constant string "UNSIGNED-PAYLOAD".
+        return "UNSIGNED-PAYLOAD"
+
+
 class HmacV1Auth(BaseSigner):
 
     # List of Query String Arguments of Interest
@@ -377,7 +473,6 @@ class HmacV1Auth(BaseSigner):
         for key in headers:
             lk = key.lower()
             if headers[key] is not  None:
-                # TODO: move hardcoded prefix to provider
                 if lk.startswith('x-amz-'):
                     custom_headers[lk] = ','.join(v.strip() for v in
                                                   headers.get_all(key))
@@ -431,7 +526,6 @@ class HmacV1Auth(BaseSigner):
 
     def get_signature(self, method, split, headers, expires=None):
         if self.credentials.token:
-            #TODO: remove hardcoded header name
             headers['x-amz-security-token'] = self.credentials.token
         string_to_sign = self.canonical_string(method,
                                                split,
@@ -460,4 +554,6 @@ AUTH_TYPE_MAPS = {
     'v3https': SigV3Auth,
     's3': HmacV1Auth,
     's3v4': S3SigV4Auth,
+    's3v4-query': S3SigV4QueryAuth,
+    'v4-query': SigV4QueryAuth,
 }
